# Generated by cpp11: do not edit by hand

read_xls_ <- function(path, sheet_i, limits, shim, col_names, col_types, na, trim_ws, guess_max, progress) {
  .Call(`_readxl_read_xls_`, path, sheet_i, limits, shim, col_names, col_types, na, trim_ws, guess_max, progress)
}

read_xlsx_ <- function(path, sheet_i, limits, shim, col_names, col_types, na, trim_ws, guess_max, progress) {
  .Call(`_readxl_read_xlsx_`, path, sheet_i, limits, shim, col_names, col_types, na, trim_ws, guess_max, progress)
}

xls_sheets <- function(path) {
  .Call(`_readxl_xls_sheets`, path)
}

xls_date_formats <- function(path) {
  .Call(`_readxl_xls_date_formats`, path)
}

xlsx_sheets <- function(path) {
  .Call(`_readxl_xlsx_sheets`, path)
}

xlsx_strings <- function(path) {
  .Call(`_readxl_xlsx_strings`, path)
}

xlsx_date_formats <- function(path) {
  .Call(`_readxl_xlsx_date_formats`, path)
}

zip_xml <- function(zip_path, file_path) {
  invisible(.Call(`_readxl_zip_xml`, zip_path, file_path))
}
